package de.tu_darmstadt.tk.SmartHomeNetworkSim.core;

import java.util.LinkedList;
import java.util.List;
import java.util.Observable;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.protocols.MQTT_protocol;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation.SimpleLink;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation.SimpleProtocol;

/**
 * Model of the smart home, which contains all the important parts of the simulation, like {@link SmartDevice} and their {@link Link}.
 *
 * @author Andreas T. Meyer-Berg
 */
public class Model extends Observable{

	private List<SmartDevice> devices;
	private List<Link> connectionNetworks;
	private List<Connection> connections;
	
	/*
	 * standard classes of the model, and user imported classes 
	 */
	private LinkedList<Class<? extends Protocol>> standardProtocols = new LinkedList<Class<? extends Protocol>>();
	private LinkedList<Class<? extends Protocol>> importedProtocols = new LinkedList<Class<? extends Protocol>>();
	
	private LinkedList<Class<? extends Link>> standardLinks = new LinkedList<Class<? extends Link>>();
	private LinkedList<Class<? extends Link>> importedLinks = new LinkedList<Class<? extends Link>>();
	
	private LinkedList<Class<? extends Connection>> standardConnections = new LinkedList<Class<? extends Connection>>();
	private LinkedList<Class<? extends Connection>> importedConnections = new LinkedList<Class<? extends Connection>>();
	
	private LinkedList<Class<? extends SmartDevice>> standardSmartDevices = new LinkedList<Class<? extends SmartDevice>>();
	private LinkedList<Class<? extends SmartDevice>> importedSmartDevices = new LinkedList<Class<? extends SmartDevice>>();
	
	/**
	 * Width of the smart home model, 0 <= device.x < width
	 */
	private int width;
	
	/**
	 * Height of the smart home model, 0 <= device.y < height
	 */
	private int height;
	
	/**
	 * Depth of the smart home model, 0 <= device.y < depth
	 */
	private int depth;
	
	/**
	 * Radius (including the middlePoinnt) of a smartDevice
	 * 
	 */
	private int device_visualization_radius = 17;
	
	/**
	 * Simulation Manager which allows simulation of this model
	 */
	private SimulationManager sim;
	
	/**
	 * Initializes the Model, with 3 default devices for testing purposes
	 */
	public Model() {
		setSim(new SimulationManager(this));
		setWidth(640);
		setHeight(480);
		setDepth(480);
		
		devices = new LinkedList<SmartDevice>();
		connectionNetworks = new LinkedList<Link>();
		connections = new LinkedList<Connection>();
		
		//Add the default Classes
		standardProtocols.add(MQTT_protocol.class);
		standardProtocols.add(SimpleProtocol.class);
		
		standardLinks.add(SimpleLink.class);
		
		standardSmartDevices.add(SmartDevice.class);
		
		standardConnections.add(ConnectionImplementation.class);
	}

	/**
	 * @return the connectionNetworks
	 */
	public List<Link> getConnectionNetworks() {
		return connectionNetworks;
	}

	/**
	 * Adds network connection
	 * @param connectionNetwork the connectionNetwork to add
	 */
	public void addConnectionNetwork(Link connectionNetwork) {
		this.connectionNetworks.add(connectionNetwork);
	}

	/**
	 * @return the devices
	 */
	public List<SmartDevice> getDevices() {
		return devices;
	}

	/**
	 * @param device the smartDevice to add to the smart home
	 */
	public void addDevices(SmartDevice device) {
		this.devices.add(device);
	}

	/**
	 * @return the width
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * @param width the width to set
	 */
	public void setWidth(int width) {
		this.width = width;
	}

	/**
	 * @return the height
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * @param height the height to set
	 */
	public void setHeight(int height) {
		this.height = height;
	}

	/**
	 * @return the depth
	 */
	public int getDepth() {
		return depth;
	}

	/**
	 * @param depth the depth to set
	 */
	public void setDepth(int depth) {
		this.depth = depth;
	}

	/**
	 * @return the device_visualization_radius
	 */
	public int getDevice_visualization_radius() {
		return device_visualization_radius;
	}

	/**
	 * @param device_visualization_radius the device_visualization_radius to set
	 */
	public void setDevice_visualization_radius(int device_visualization_radius) {
		this.device_visualization_radius = device_visualization_radius;
	}
	
	/**
	 * @return the connections
	 */
	public List<Connection> getConnections() {
		return connections;
	}

	/**
	 * @param connection the connection to add
	 */
	public void addConnection(Connection connection) {
		this.connections.add(connection);
	}	
	
	public void setChanged(){
		super.setChanged();
	}

	/**
	 * @return the sim
	 */
	public SimulationManager getSim() {
		return sim;
	}

	/**
	 * @param sim the sim to set
	 */
	public void setSim(SimulationManager sim) {
		this.sim = sim;
	}
	
	/**
	 * Returns Protocol Classes of the Model
	 * @return available Protocol Classes
	 */
	public LinkedList<Class<? extends Protocol>> getProtocolClasses(){
		LinkedList<Class<? extends Protocol>> export = new LinkedList<Class<? extends Protocol>>();
		export.addAll(standardProtocols);
		export.addAll(importedProtocols);
		return export;
	}
	
	/**
	 * Adds newProtocol to the available protocol classes
	 * @param newProtocol new Protocol Class to be added
	 */
	public void addProtocolClass(Class<? extends Protocol> newProtocol){
		importedProtocols.add(newProtocol);
	}
	
	/**
	 * Removes Protocol from the available protocol classes
	 * @param remove protocol to be removed
	 */
	public void removeProtocolClass(Class<? extends Protocol> remove){
		importedProtocols.remove(remove);
	}
	
	/**
	 * Returns Link Classes of the Model
	 * @return available Link classes
	 */
	public LinkedList<Class<? extends Link>> getLinkClasses(){
		LinkedList<Class<? extends Link>> export = new LinkedList<Class<? extends Link>>();
		export.addAll(standardLinks);
		export.addAll(importedLinks);
		return export;
	}
	
	/**
	 * Adds newLink to the available Link classes
	 * @param newLink new Link class to be added
	 */
	public void addLinkClass(Class<? extends Link> newLink){
		importedLinks.add(newLink);
	}
	
	/**
	 * Removes Link from the available Link classes
	 * @param remove Link Class to be removed
	 */
	public void removeLinkClass(Class<? extends Link> remove){
		importedLinks.remove(remove);
	}
	
	/**
	 * Returns Connection Classes of the Model
	 * @return available Connections classes
	 */
	public LinkedList<Class<? extends Connection>> getConnectionClasses(){
		LinkedList<Class<? extends Connection>> export = new LinkedList<Class<? extends Connection>>();
		export.addAll(standardConnections);
		export.addAll(importedConnections);
		return export;
	}
	
	/**
	 * Adds Connection Class to the available Connection Classes
	 * @param newConnection new Connection to be added
	 */
	public void addConnectionClass(Class<? extends Connection> newConnection){
		importedConnections.add(newConnection);
	}
	
	/**
	 * Removes Connection Class from the available Connection Classes
	 * @param remove Connection Class to be removed
	 */
	public void removeConnectionClass(Class<? extends Connection> remove){
		importedConnections.remove(remove);
	}
	
	/**
	 * Returns SmartDevice Classes of the Model
	 * @return available SmartDevice Classes
	 */
	public LinkedList<Class<? extends SmartDevice>> getSmartDeviceClasses(){
		LinkedList<Class<? extends SmartDevice>> export = new LinkedList<Class<? extends SmartDevice>>();
		export.addAll(standardSmartDevices);
		export.addAll(importedSmartDevices);
		return export;
	}
	
	/**
	 * Adds newSmartDevice Class to the available SmartDevice Classes
	 * @param newSmartDevice new SmartDevice Class to be added
	 */
	public void addSmartDeviceClass(Class<? extends SmartDevice> newSmartDevice){
		importedSmartDevices.add(newSmartDevice);
	}
	
	/**
	 * Removes SmartDevice Class from the available SmartDevice Classes
	 * @param remove SmartDevice Class to be removed
	 */
	public void removeSmartDeviceClass(Class<? extends SmartDevice> remove){
		importedSmartDevices.remove(remove);
	}
}
